// Copyright 2017 by Jon Dart.  All Rights Reserved.

// Utility to filter out EPD records with mate scores or very large scores

#include "board.h"
#include "boardio.h"
#include "legal.h"
#include "movegen.h"
#include "options.h"
#include "movearr.h"
#include "notation.h"
#include "chessio.h"
#include "movearr.h"
#include "globals.h"
#include "util.h"
#include "chessio.h"
#include "search.h"

extern "C"
{
#include <stddef.h>
#include <string.h>
#include <ctype.h>
};
#include <fstream>
#include <iostream>
#include <ctype.h>

using namespace std;

static const int maxScore = 20*PAWN_VALUE;

static void show_usage()
{
   cerr << "Usage: epdfilter epd_file" << endl;
}

static int score(SearchController *searcher,const Board &board,
                 Statistics &stats) 
{
   stats.clear();
   searcher->findBestMove(board,
                          FixedDepth,
                          999,
                          0,            /* extra time allowed */
                          6,            /* ply limit */
                          false,        /* background */
                          false, /* UCI */
                          stats,
                          Silent);
                     
   return stats.value;
}

int CDECL main(int argc, char **argv)
{
   Bitboard::init();
   Attacks::init();
   Scoring::init();
   if (!initGlobals(argv[0], false)) {
      cleanupGlobals();
      exit(-1);
   }
   atexit(cleanupGlobals);
   
   Board board;
   int arg = 1;

   if (argc <=1)
   {
      show_usage();
      return -1;
   }
   else
   {
      SearchController *searcher = new SearchController();
      Statistics stats;

      ifstream epd_file( argv[arg], ios::in | ios::binary);
      if (!epd_file.good()) {
         cerr << "could not open file " << argv[arg] << endl;
         exit(-1);
      }
      else
      {
         while (!epd_file.eof())
         {
            EPDRecord rec;
            if (ChessIO::readEPDRecord(epd_file,board,rec)) {
               Bitboard atcks =
                  board.calcAttacks(board.kingSquare(board.oppositeSide()),board.sideToMove());
               if (atcks.isClear() && Util::Abs(score(searcher,board,stats)) < maxScore) {
                  ChessIO::writeEPDRecord(cout,board,rec);
               }
            }
         }
         epd_file.close();
      }
      delete searcher;
   }

   return 0;
}
